import setuptools

setuptools.setup(
    name="kindenty_base",
    version="0.1.4",
    author="kindenty",
    author_email="kindenty@163.com",
    description="测试",
    long_description="text/markdown",
    url="https://gitee.com/kindenty-yueyu/cats-strategy-core",
    packages=setuptools.find_packages(),
    classifiers=["Programming Language :: Python :: 3"],
)
